@@ -16,9 +16,9 @@
 //! Functions and data structures of the swayrd demon.
 
 use crate::cmds;
-use crate::con;
 use crate::config;
 use crate::layout;
+use crate::tree as t;
 use crate::util;
 use std::collections::HashMap;
 use std::io::Read;
@@ -30,7 +30,7 @@ use std::time::{SystemTime, UNIX_EPOCH};
 use swayipc as s;
 
 pub fn run_demon() {
-    let extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>> =
+    let extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>> =
         Arc::new(RwLock::new(HashMap::new()));
     let extra_props_for_ev_handler = extra_props.clone();
 
@@ -42,31 +42,41 @@ pub fn run_demon() {
 }
 
 fn connect_and_subscribe() -> s::Fallible<s::EventStream> {
-    s::Connection::new()?
-        .subscribe(&[s::EventType::Window, s::EventType::Workspace])
+    s::Connection::new()?.subscribe(&[
+        s::EventType::Window,
+        s::EventType::Workspace,
+        s::EventType::Shutdown,
+    ])
 }
 
 pub fn monitor_sway_events(
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) {
     let config = config::load_config();
+    let mut resets = 0;
+    let max_resets = 10;
 
     'reset: loop {
+        if resets >= max_resets {
+            break;
+        }
+        resets += 1;
+
         println!("Connecting to sway for subscribing to events...");
         match connect_and_subscribe() {
             Err(err) => {
                 eprintln!("Could not connect and subscribe: {}", err);
                 std::thread::sleep(std::time::Duration::from_secs(3));
-                break 'reset;
             }
             Ok(iter) => {
                 for ev_result in iter {
-                    let handled;
+                    let show_extra_props_state;
+                    resets = 0;
                     match ev_result {
                         Ok(ev) => match ev {
                             s::Event::Window(win_ev) => {
                                 let extra_props_clone = extra_props.clone();
-                                handled = handle_window_event(
+                                show_extra_props_state = handle_window_event(
                                     win_ev,
                                     extra_props_clone,
                                     &config,
@@ -74,23 +84,30 @@ pub fn monitor_sway_events(
                             }
                             s::Event::Workspace(ws_ev) => {
                                 let extra_props_clone = extra_props.clone();
-                                handled = handle_workspace_event(
+                                show_extra_props_state = handle_workspace_event(
                                     ws_ev,
                                     extra_props_clone,
                                 );
                             }
-                            _ => handled = false,
+                            s::Event::Shutdown(sd_ev) => {
+                                println!(
+                                    "Sway shuts down with reason '{:?}'.",
+                                    sd_ev.change
+                                );
+                                break 'reset;
+                            }
+                            _ => show_extra_props_state = false,
                         },
                         Err(e) => {
                             eprintln!("Error while receiving events: {}", e);
                             std::thread::sleep(std::time::Duration::from_secs(
                                 3,
                             ));
+                            show_extra_props_state = false;
                             eprintln!("Resetting!");
-                            break 'reset;
                         }
                     }
-                    if handled {
+                    if show_extra_props_state {
                         println!(
                             "New extra_props state:\n{:#?}",
                             *extra_props.read().unwrap()
@@ -100,11 +117,12 @@ pub fn monitor_sway_events(
             }
         }
     }
+    println!("Swayr demon shutting down.")
 }
 
 fn handle_window_event(
     ev: Box<s::WindowEvent>,
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
     config: &config::Config,
 ) -> bool {
     let s::WindowEvent {
@@ -143,7 +161,7 @@ fn handle_window_event(
 
 fn handle_workspace_event(
     ev: Box<s::WorkspaceEvent>,
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) -> bool {
     let s::WorkspaceEvent {
         change,
@@ -176,7 +194,7 @@ fn handle_workspace_event(
 
 fn update_last_focus_time(
     id: i64,
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) {
     let mut write_lock = extra_props.write().unwrap();
     if let Some(wp) = write_lock.get_mut(&id) {
@@ -184,8 +202,9 @@ fn update_last_focus_time(
     } else {
         write_lock.insert(
             id,
-            con::ExtraProps {
+            t::ExtraProps {
                 last_focus_time: get_epoch_time_as_millis(),
+                last_focus_time_for_next_prev_seq: 0,
             },
         );
     }
@@ -193,7 +212,7 @@ fn update_last_focus_time(
 
 fn remove_extra_props(
     id: i64,
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) {
     extra_props.write().unwrap().remove(&id);
 }
@@ -206,7 +225,7 @@ fn get_epoch_time_as_millis() -> u128 {
 }
 
 pub fn serve_client_requests(
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) {
     match std::fs::remove_file(util::get_swayr_socket_path()) {
         Ok(()) => println!("Deleted stale socket from previous run."),
@@ -235,7 +254,7 @@ pub fn serve_client_requests(
 
 fn handle_client_request(
     mut stream: UnixStream,
-    extra_props: Arc<RwLock<HashMap<i64, con::ExtraProps>>>,
+    extra_props: Arc<RwLock<HashMap<i64, t::ExtraProps>>>,
 ) {
     let mut cmd_str = String::new();
     if stream.read_to_string(&mut cmd_str).is_ok() {
