@@ -174,9 +174,9 @@ pub struct Tree<'a> {
 
 #[derive(Copy, Clone, PartialEq, Eq)]
 enum IndentLevel {
-    Fixed(u32),
+    Fixed(usize),
     WorkspacesZeroWindowsOne,
-    TreeDepth(u8),
+    TreeDepth(usize),
 }
 
 pub struct DisplayNode<'a> {
@@ -196,12 +196,16 @@ impl<'a> Tree<'a> {
         self.id_parent.get(&id).map(|pid| self.get_node_by_id(*pid))
     }
 
-    pub fn get_workspace_node(&self, id: i64) -> Option<&&s::Node> {
+    pub fn get_parent_node_of_type(
+        &self,
+        id: i64,
+        t: Type,
+    ) -> Option<&&s::Node> {
         let n = self.get_node_by_id(id);
-        if n.get_type() == Type::Workspace {
+        if n.get_type() == t {
             Some(n)
         } else if let Some(pid) = self.id_parent.get(&id) {
-            self.get_workspace_node(*pid)
+            self.get_parent_node_of_type(*pid, t)
         } else {
             None
         }
@@ -252,6 +256,15 @@ impl<'a> Tree<'a> {
             .expect("No current Workspace")
     }
 
+    pub fn get_outputs(&self) -> Vec<DisplayNode> {
+        let outputs: Vec<&s::Node> = self
+            .root
+            .iter()
+            .filter(|n| n.get_type() == Type::Output && !n.is_scratchpad())
+            .collect();
+        self.as_display_nodes(&outputs, IndentLevel::Fixed(0))
+    }
+
     pub fn get_workspaces(&self) -> Vec<DisplayNode> {
         let mut v = self.sorted_nodes_of_type(Type::Workspace);
         if !v.is_empty() {
@@ -331,6 +344,19 @@ impl<'a> Tree<'a> {
         }
     }
 
+    pub fn get_outputs_workspaces_containers_and_windows(
+        &self,
+    ) -> Vec<DisplayNode> {
+        let outputs = self.sorted_nodes_of_type(Type::Output);
+        let v: Rc<RefCell<Vec<&s::Node>>> = Rc::new(RefCell::new(vec![]));
+        for o in outputs {
+            self.push_subtree_sorted(o, Rc::clone(&v));
+        }
+
+        let x = self.as_display_nodes(&*v.borrow(), IndentLevel::TreeDepth(1));
+        x
+    }
+
     pub fn get_workspaces_containers_and_windows(&self) -> Vec<DisplayNode> {
         let workspaces = self.sorted_nodes_of_type(Type::Workspace);
         let v: Rc<RefCell<Vec<&s::Node>>> = Rc::new(RefCell::new(vec![]));
@@ -440,7 +466,7 @@ impl DisplayFormat for DisplayNode<'_> {
 
         let fmt = match self.node.get_type() {
             Type::Root => String::from("Cannot format Root"),
-            Type::Output => String::from("Cannot format Output"),
+            Type::Output => cfg.get_format_output_format(),
             Type::Workspace => cfg.get_format_workspace_format(),
             Type::Container => cfg.get_format_container_format(),
             Type::Window => cfg.get_format_window_format(),
@@ -476,12 +502,12 @@ impl DisplayFormat for DisplayNode<'_> {
                 &maybe_html_escape(html_escape, self.node.get_app_name()),
             )
             .replace(
-                "{workspace_name}",
+                "{output_name}",
                 &maybe_html_escape(
                     html_escape,
                     self.tree
-                        .get_workspace_node(self.node.id)
-                        .map_or("<no_workspace>", |w| w.get_name()),
+                        .get_parent_node_of_type(self.node.id, Type::Output)
+                        .map_or("<no_output>", |w| w.get_name()),
                 ),
             )
             .replace(
@@ -489,7 +515,7 @@ impl DisplayFormat for DisplayNode<'_> {
                 &maybe_html_escape(
                     html_escape,
                     self.tree
-                        .get_workspace_node(self.node.id)
+                        .get_parent_node_of_type(self.node.id, Type::Workspace)
                         .map_or("<no_workspace>", |w| w.get_name()),
                 ),
             )
@@ -537,7 +563,11 @@ impl DisplayFormat for DisplayNode<'_> {
                     depth += 1;
                     node = p;
                 }
-                depth - offset as usize
+                if offset > depth {
+                    0
+                } else {
+                    depth - offset as usize
+                }
             }
         }
     }
